<?php

/**
 * @author     M2E Pro Developers Team
 * @copyright  M2E LTD
 * @license    Commercial use is forbidden
 */

namespace Ess\M2ePro\Controller\Adminhtml\Walmart\Listing\Unmanaged;

use Ess\M2ePro\Block\Adminhtml\Walmart\Listing\Product\Add\SourceMode as SourceModeBlock;

class MoveToListing extends \Ess\M2ePro\Controller\Adminhtml\Walmart\Main
{
    /** @var \Ess\M2ePro\Helper\Data\Session */
    private $sessionHelper;

    public function __construct(
        \Ess\M2ePro\Helper\Data\Session $sessionHelper,
        \Ess\M2ePro\Model\ActiveRecord\Component\Parent\Walmart\Factory $walmartFactory,
        \Ess\M2ePro\Controller\Adminhtml\Context $context
    ) {
        parent::__construct($walmartFactory, $context);

        $this->sessionHelper = $sessionHelper;
    }

    public function execute()
    {
        $sessionKey = \Ess\M2ePro\Helper\Component\Walmart::NICK . '_'
            . \Ess\M2ePro\Helper\View::MOVING_LISTING_OTHER_SELECTED_SESSION_KEY;
        $selectedProducts = $this->sessionHelper->getValue($sessionKey);

        /** @var \Ess\M2ePro\Model\Listing $listingInstance */
        $listingInstance = $this->walmartFactory->getCachedObjectLoaded(
            'Listing',
            (int)$this->getRequest()->getParam('listingId')
        );

        $errorsCount = 0;
        $tempProducts = [];
        foreach ($selectedProducts as $otherListingProduct) {
            /** @var \Ess\M2ePro\Model\Listing\Other $listingOther */
            $listingOther = $this->walmartFactory->getObjectLoaded('Listing\Other', $otherListingProduct);

            $listingProduct = $listingInstance->getChildObject()->addProductFromOther(
                $listingOther,
                \Ess\M2ePro\Helper\Data::INITIATOR_USER
            );

            if (!($listingProduct instanceof \Ess\M2ePro\Model\Listing\Product)) {
                $errorsCount++;
                continue;
            }

            $tempProducts[] = $listingProduct->getId();
        }

        $addingProducts = array_unique(
            array_merge(
                $tempProducts,
                $listingInstance->getSetting('additional_data', 'adding_listing_products_ids', [])
            )
        );

        $listingInstance->setSetting('additional_data', 'adding_listing_products_ids', $addingProducts);
        $listingInstance->setSetting('additional_data', 'source', SourceModeBlock::MODE_OTHER);
        $listingInstance->save();

        $this->sessionHelper->removeValue($sessionKey);

        if ($errorsCount) {
            if (count($selectedProducts) == $errorsCount) {
                $this->setJsonContent(
                    [
                        'result' => false,
                        'message' => __(
                            'Products were not moved because they already exist in the selected Listing or do not
                            belong to the channel account or marketplace of the listing'
                        ),
                    ]
                );

                return $this->getResult();
            }

            $this->setJsonContent(
                [
                    'result' => true,
                    'isFailed' => true,
                    'message' => __(
                        'Some products were not moved because they already exist in the selected Listing or do not
                        belong to the channel account or marketplace of the listing.'
                    ),
                ]
            );
        } else {
            $this->setJsonContent(
                [
                    'result' => true,
                    'message' => __('Product(s) was Moved.'),
                ]
            );
        }

        return $this->getResult();
    }
}
